@@ -29,12 +29,6 @@
 
 #include <sys/types.h>
 
-typedef struct {
-    char **t; /* the text rows */
-    int n;    /* number of rows */
-    int m;    /* maximum row length */
-} TextRows;
-
 
 /* Boolean options */
 #define NOLOGO_BOOL_OPTION                      0
@@ -48,10 +42,9 @@ typedef struct {
 #define NO_POWER_CONNECTOR_CHECK_BOOL_OPTION    8
 #define ALLOW_GLX_WITH_COMPOSITE_BOOL_OPTION    10
 #define RANDR_ROTATION_BOOL_OPTION              11
-#define TWINVIEW_BOOL_OPTION                    12
 #define SEPARATE_X_SCREENS_BOOL_OPTION          13
 #define XINERAMA_BOOL_OPTION                    14
-#define NO_TWINVIEW_XINERAMA_INFO_BOOL_OPTION   15
+#define NVIDIA_XINERAMA_INFO_BOOL_OPTION        15
 #define NOFLIP_BOOL_OPTION                      16
 #define DAC_8BIT_BOOL_OPTION                    17
 #define USE_EDID_FREQS_BOOL_OPTION              18
@@ -107,7 +100,6 @@ typedef struct __options {
     int force_generate;
     int tree;
     int post_tree;
-    int silent;
     int keyboard_list;
     int mouse_list;
     int enable_all_gpus;
@@ -115,6 +107,7 @@ typedef struct __options {
     int disable_scf;
     int query_gpu_info;
     int preserve_driver;
+    int restore_original_backup;
     
     /*
      * the option parser will set bits in boolean_options to indicate
@@ -146,9 +139,9 @@ typedef struct __options {
     char *nvidia_cfg_path;
     char *extract_edids_from_file;
     char *extract_edids_output_file;
-    char *twinview_xinerama_info_order;
+    char *nvidia_xinerama_info_order;
     char *logo_path;
-    char *twinview_orientation;
+    char *metamode_orientation;
     char *use_display_device;
     char *custom_edid;
     char *tv_standard;
@@ -208,24 +201,6 @@ int copy_file(const char *srcfile, const char *dstfile, mode_t mode);
 
 int directory_exists(const char *dir);
 
-void reset_current_terminal_width(unsigned short new_val);
-
-TextRows *nv_format_text_rows(const char *prefix,
-                              const char *str,
-                              int width, int word_boundary);
-void nv_text_rows_append(TextRows *t, const char *msg);
-void nv_free_text_rows(TextRows *t);
-void nv_concat_text_rows(TextRows *t0, TextRows *t1);
-
-char *fget_next_line(FILE *fp, int *eof);
-
-void fmtout(const char *fmt, ...);
-void fmtoutp(const char *prefix, const char *fmt, ...);
-void fmtmsg(const char *fmt, ...);
-void fmterr(const char *fmt, ...);
-void fmtwarn(const char *fmt, ...);
-
-
 /* make_usable.c */
 
 int update_modules(XConfigPtr config);
